Imports System

Imports CenterSpace.NMath.Core

Namespace CenterSpace.NMath.Core.Examples.VisualBasic

  ' A .NET example in Visual Basic showing how to solve a linear system with simplex method and
  ' linear programming.
  Module LinearProgrammingExample

    Sub Main()

      ' A farmer has 640 acres of farmland. It can be planted with wheat, barley, corn Or a
      ' combination of the three. The farmer wishes to maximize his profit subject to the
      ' limits on land, fertilizer, And water.

      ' Currently, wheat Is $3.38/bushel. The farmer can expect a yield of 55 bushels/acre.
      Dim WheatPrice As Double = 3.38
      Dim WheatYield As Double = 55.0
      Dim WheatRevenuePerAcre As Double = WheatPrice * WheatYield

      ' Currently, barley Is $1.98/bushel. The farmer can expect a yield of 75 bushels/acre.
      Dim BarleyPrice As Double = 1.98
      Dim BarleyYield As Double = 75.0
      Dim BarleyRevenuePerAcre As Double = BarleyPrice * BarleyYield

      ' Currently, corn Is $1.70/bushel. The farmer can expect a yield of 110 bushels/acre.
      Dim CornPrice As Double = 1.7
      Dim CornYield As Double = 110.0
      Dim CornRevenuePerAcre As Double = CornPrice * CornYield

      Console.WriteLine()

      ' Therefore, the objective function is
      Console.Write("Maximize " & WheatRevenuePerAcre & "w + ")
      Console.WriteLine(BarleyRevenuePerAcre & "b + " & CornRevenuePerAcre & "c")
      Console.WriteLine("where")
      Console.WriteLine()

      Dim Revenue As New DoubleVector(WheatRevenuePerAcre, BarleyRevenuePerAcre, CornRevenuePerAcre)
      Dim LPProblem As New LinearProgrammingProblem(Revenue)

      ' The farmer has 8,000 lbs of nitrogen fertilizer. It's known that wheat requires
      ' 12 lb/acre, barley 5 lb/acre And corn 22 lb/acre.
      Console.WriteLine("12w + 5b + 22c <= 8000")
      LPProblem.AddUpperBoundConstraint(New DoubleVector(12.0, 5.0, 22.0), 8000.0)

      ' The farmer has 22,000 lbs of phosphate fertilizer. It's known that wheat requires
      ' 30 lb/acre, barley 8 lb/acre And corn 50 lb/acre.
      Console.WriteLine("30w + 8b + 50c <= 22000")
      LPProblem.AddUpperBoundConstraint(New DoubleVector(30.0, 8.0, 50.0), 22000.0)

      ' The farmer has a permit for 1,000 acre-feet of water. Wheat requires 1.5 ft of water, 
      ' barley requires 0.7 And corn 2.2.
      Console.WriteLine("1.5w + 0.7b + 2.2c <= 1200")
      LPProblem.AddUpperBoundConstraint(New DoubleVector(1.5, 0.7, 2.2), 1200.0)

      ' The farmer has a maximum of 640 acres for planting.
      Console.WriteLine("w + b + c <= 640")
      LPProblem.AddUpperBoundConstraint(New DoubleVector(1.0, 1.0, 1.0), 640.0)
      Dim I As Integer = 0
      For I = 0 To Revenue.Length - 1
        LPProblem.AddLowerBound(I, 0.0)
      Next

      ' Create an LP solver
      Dim Solver As New PrimalSimplexSolverORTools()

      ' Solve
      Solver.Solve(LPProblem)

      ' Was a finite solution found?
      Console.WriteLine()
      If Solver.Result = PrimalSimplexSolverORTools.SolveResult.Optimal Then
        Console.WriteLine("solution: " & Solver.OptimalX.ToString("f0"))
        Console.WriteLine()
        Console.WriteLine("optimal value: " & Solver.OptimalObjectiveFunctionValue.ToString("f0"))
      End If
      Console.WriteLine()

      ' Let's say the farmer is also contractually obligated to farm at least 10 acres
      ' of barley.
      Console.WriteLine("Add variable bound: b >= 10")
      LPProblem.AddLowerBound(1, 10.0)

      ' Solve again
      Solver.Solve(LPProblem)

      ' Good?
      Console.WriteLine()
      If Solver.Result = PrimalSimplexSolverORTools.SolveResult.Optimal Then
        Console.WriteLine("solution: " + Solver.OptimalX.ToString("f0"))
        Console.WriteLine()
        Console.WriteLine("optimal value: " + Solver.OptimalObjectiveFunctionValue.ToString("f0"))
      End If

      Console.WriteLine()
      Console.WriteLine("Press Enter Key")
      Console.Read()

    End Sub

  End Module

End Namespace

